@@ -249,7 +249,8 @@ impl Expr {
                 x => joker(x)
             }
             Sel(a, b) => match (a.eval(closed), b.eval(closed)) {
-                (Jok(x), Jok(y)) => joker(sel(*x, *y)).eval(closed),
+                (Jok(x), Jok(y)) if not((&*x).clone()).eval(closed) == *y =>
+                    joker(sel(*x, *y)).eval(closed),
                 (Seq(x1, x2), _0) if closed && *x1 == _0 =>
                     seq(_0, sel(*x2, joker(_1))).eval(closed),
                 (_0, Seq(y1, y2)) if closed && *y1 == _0 =>
@@ -264,11 +265,11 @@ impl Expr {
                 (x, y) => sel(x, y)
             }
             Seq(a, b) => match (a.eval(closed), b.eval(closed)) {
-                (_0, _0) => _0,
-                (_1, _1) => _1,
+                (_0, _0) if closed => _0,
+                (_1, _1) if closed => _1,
                 (_0, x) => {
                     match x {
-                        Seq(x, y) if *x == _0 => seq(_0, *y),
+                        Seq(x, y) if closed && *x == _0 => seq(_0, *y),
                         Jok(y) if closed && *y == _1 => _0,
                         Sel(y, z) if closed => {
                             match (*y, *z) {
@@ -282,7 +283,7 @@ impl Expr {
                 }
                 (_1, x) => {
                     match x {
-                        Seq(x, y) if *x == _1 => seq(_1, *y),
+                        Seq(x, y) if closed && *x == _1 => seq(_1, *y),
                         Jok(y) if closed && *y == _0 => _1,
                         Sel(y, z) if closed => {
                             match (*y, *z) {
@@ -376,25 +377,32 @@ impl Expr {
         self.swap(closed) == self.eval(closed)
     }
 
-    /// Returns `true` if the double-not does not evaluate to itself with Open Joker Calculus.
-    pub fn divergent(&self) -> bool {
-        not(not(self.clone())).eval_open() != self.eval_open()
-    }
+    /// Returns `true` if expression is two-sided with Open Joker Calculus.
+    pub fn two_sided_open(&self) -> bool {self.two_sided(false)}
+    /// Returns `true` if expression is two-sided with Closed Joker Calculus.
+    pub fn two_sided_closed(&self) -> bool {self.two_sided(true)}
 
-    /// Returns `true` if the expression is contracting with Open Joker Calculus.
-    pub fn contracting_open(&self) -> bool {self.contracting(false)}
-    /// Returns `true` if the expression is contracting with Closed Joker Calculus.
-    pub fn contracting_closed(&self) -> bool {self.contracting(true)}
-
-    /// Returns `true` if the sequence repetition of itself evaluates to itself.
+    /// Returns `true` if expression is two-sided.
     ///
     /// When `closed` is set to `true`,
     /// the variant Closed Joker Calculus is used.
     ///
     /// When `closed` is set to `false`,
     /// the variant Open Joker Calculus is used.
-    pub fn contracting(&self, closed: bool) -> bool {
-        seq(self.clone(), self.clone()).eval(closed) == self.eval(closed)
+    pub fn two_sided(&self, closed: bool) -> bool {
+        !self.one_sided(closed)
+    }
+
+    /// Returns `true` if the double-not does not evaluate to itself with Open Joker Calculus.
+    pub fn divergent(&self) -> bool {
+        not(not(self.clone())).eval_open() != self.eval_open()
+    }
+
+    /// Returns `true` if the sequence repetition of itself evaluates to itself in Closed variant.
+    ///
+    /// There are no contracting expressions in the Open variant.
+    pub fn contracting(&self) -> bool {
+        seq(self.clone(), self.clone()).eval_closed() == self.eval_closed()
     }
 
     /// Returns `true` if expression contains a joker.
@@ -500,23 +508,27 @@ mod tests {
         assert_eq!(a.eval_open(), seshatic(platonism()));
 
         let a = platonic(platonism());
-        assert_eq!(a.eval_open(), platonism());
+        assert_eq!(a.eval_open(), platonic(platonism()));
+        assert_eq!(a.eval_closed(), platonism());
 
         let a = seq(joker(seshatism()), platonism());
-        assert_eq!(a.eval_open(), sel(seshatic(platonism()), platonism()));
+        assert_eq!(a.eval_open(), sel(seshatic(platonism()), platonic(platonism())));
+        assert_eq!(a.eval_closed(), sel(seshatic(platonism()), platonism()));
 
         let a = joker(seq(joker(seshatism()), platonism()));
-        assert_eq!(a.eval_open(), joker(sel(seshatic(platonism()), platonism())));
+        assert_eq!(a.eval_open(), joker(sel(seshatic(platonism()), platonic(platonism()))));
+        assert_eq!(a.eval_closed(), joker(sel(seshatic(platonism()), platonism())));
 
         let a = seq(sel(seshatism(), joker(platonism())), seshatism());
-        assert_eq!(a.eval_open(), sel(seshatism(), sel(platonic(seshatism()), seshatism())));
+        assert_eq!(a.eval_open(), sel(seshatic(seshatism()), sel(platonic(seshatism()), seshatic(seshatism()))));
+        assert_eq!(a.eval_closed(), sel(seshatism(), sel(platonic(seshatism()), seshatism())));
 
         let a = sel(seshatic(platonism()), platonic(seshatism()));
         assert_eq!(a.eval_open(), sel(seshatic(platonism()), platonic(seshatism())));
 
-        let a = sel(joker(seshatism()), joker(platonism()));
-        assert_eq!(a.eval_open(), joker(joker(seshatism())));
-        assert_eq!(a.eval_closed(), seshatism());
+        let a = jc!(?1, ?0);
+        assert_eq!(a.eval_open(), jc!(??1));
+        assert_eq!(a.eval_closed(), jc!(1));
 
         let a = sel(platonism(), joker(seshatism()));
         assert_eq!(a.eval_open(), sel(platonism(), joker(seshatism())));
@@ -529,7 +541,7 @@ mod tests {
         assert_eq!(a.eval_closed(), seshatic(platonism()));
 
         let a = not(seshatic(seshatism()));
-        assert_eq!(a.eval_open(), platonism());
+        assert_eq!(a.eval_open(), platonic(joker(seshatism())));
         assert_eq!(a.eval_closed(), platonism());
 
         let a = seq(sel(seshatism(), seshatism()), platonism());
@@ -541,6 +553,7 @@ mod tests {
         assert_eq!(a.eval_closed(), seshatism());
 
         let a = not(seshatic(joker(platonism())));
+        assert_eq!(a.eval_open(), platonic(joker(joker(platonism()))));
         assert_eq!(a.eval_closed(), platonism());
 
         let a = seq(joker(seshatism()), joker(joker(platonism())));
@@ -582,20 +595,20 @@ mod tests {
         assert_eq!(a.eval_closed(), seshatic(platonism()));
 
         let a = seshatic(seshatic(platonism()));
-        assert_eq!(a.eval_open(), seshatic(platonism()));
+        assert_eq!(a.eval_open(), seshatic(seshatic(platonism())));
         assert_eq!(a.eval_closed(), seshatic(platonism()));
 
         let a = seshatic(seshatic(seshatism()));
-        assert_eq!(a.eval_open(), seshatism());
+        assert_eq!(a.eval_open(), seshatic(seshatic(seshatism())));
         assert_eq!(a.eval_closed(), seshatism());
 
         let a = seshatic(sel(seshatic(platonism()), seshatic(joker(seshatism()))));
-        assert_eq!(a.eval_open(), seshatic(sel(platonism(), joker(seshatism()))));
+        assert_eq!(a.eval_open(), seshatic(seshatic(sel(platonism(), joker(seshatism())))));
         assert_eq!(a.eval_closed(), seshatic(sel(platonism(), joker(seshatism()))));
 
-        let a = sel(not(seshatic(platonism())), not(seshatic(joker(seshatism()))));
-        assert_eq!(a.eval_open(), platonic(joker(sel(platonism(), joker(seshatism())))));
-        assert_eq!(a.eval_closed(), platonic(sel(joker(platonism()), seshatism())));
+        let a = jc!((!(1 0), !(1 ?1)));
+        assert_eq!(a.eval_open(), jc!(0 (?0, ??1)));
+        assert_eq!(a.eval_closed(), jc!(0 (?0, 1)));
 
         let a = joker(sel(platonism(), joker(seshatism())));
         assert_eq!(a.eval_open(), joker(sel(platonism(), joker(seshatism()))));
@@ -614,6 +627,21 @@ mod tests {
         assert_eq!(a.eval_closed(), seshatic(sel(platonism(), joker(seshatism()))));
     }
 
+    #[test]
+    fn test_joker_ambiguity() {
+        let a = jc!(0, 1);
+        assert_eq!(a.eval_closed(), jc!(?0));
+        assert_eq!(a.eval_open(), jc!(?0));
+
+        let a = jc!(0, 1 ?0);
+        assert_eq!(a.eval_closed(), jc!(?0));
+        assert_eq!(a.eval_open(), jc!(0, 1 ?0));
+
+        let a = jc!(!(1 ?1), 1 ?1);
+        assert_eq!(a.eval_closed(), jc!(?(0 1)));
+        assert_eq!(a.eval_open(), jc!(0 ??1, 1 ?1));
+    }
+
     #[test]
     fn test_depth_surface() {
         let a = seshatism();
